SUBMERCADOS = ["SUDESTE", "SUL", "NORDESTE", "NORTE"]
REES = [
    "SUDESTE",
    "SUL",
    "NORDESTE",
    "NORTE",
    "ITAIPU",
    "MADEIRA",
    "TPIRES",
    "BMONTE",
    "MAN-AP",
    "PARANA",
    "IGUACU",
    "PRNPANEMA",
]
NUM_CENARIOS = 2000
MAX_ANOS_ESTUDO = 10
MAX_ANOS_HISTORICO = 120
MAX_CONFIGURACOES = 60
MAX_ITERS = 200
MAX_UHES = 300
MAX_UTES = 300
MAX_RES = 100
MAX_REES = 30
NUM_PATAMARES = 3
MAX_LAG_ADTERM = 2
NUM_CONFIGS_DGER = 100
NUM_VARIAVEIS_MEDIAS = 52
NUM_VARIAVEIS_CUSTO_PMO = 16
ORDEM_MAX_PARP = 6
MESES = [
    "JANEIRO",
    "FEVEREIRO",
    "MARCO",
    "ABRIL",
    "MAIO",
    "JUNHO",
    "JULHO",
    "AGOSTO",
    "SETEMBRO",
    "OUTUBRO",
    "NOVEMBRO",
    "DEZEMBRO",
]
MESES_DF = [
    "Janeiro",
    "Fevereiro",
    "Março",
    "Abril",
    "Maio",
    "Junho",
    "Julho",
    "Agosto",
    "Setembro",
    "Outubro",
    "Novembro",
    "Dezembro",
]
MAX_MESES_ESTUDO = MAX_ANOS_ESTUDO * len(MESES)
